#Setting the working directory ----
#(so R knows where to look for data and save files)
setwd("Where your data is")


#Installing and loading the necessary packages ----
# #(so R can access the any additional functions needed for the next steps)
library(dplyr) 
library(readxl)
library(ggplot2)
library(tidyr)
library(stringr)
#we could also just load tidyverse and readxl. readxl is included when installing the tidyverse but isn't loaded with library(tidyverse) 

#Importing, exploring and cleaning the data ----
#(to get the data in R an make sure the data is error free for analysis)


US_unemployment<-read_excel("Unemployment.xls")
US_poverty<-read_excel("PovertyEstimates.xls")
  
#reading in with default settings brings in background information 
#we don't need. Need to skip the first 6 rows for unemployment
# and also skip 4 for poverty so rows are recognized

US_unemployment<-read_excel("Unemployment.xls", skip = 6)
US_poverty<-read_excel("PovertyEstimates.xls",skip=4)

#notice this reads in the first sheet from the file by default
#However, we can also use sheet=2 to get the variable descriptions


#As we're interested in looking at data from both sets we 
#need to 'merge' the data. There are different ways to do this
#in, but tidyverse using 'joins' which use matching columns
#to add columns from one dataset to another
#https://dplyr.tidyverse.org/reference/join.html
#to start, we can see common names using: 
names(US_poverty) %in% names(US_unemployment)

#which shows the first two column names of the US poverty data
#is in the US unemployment dataset. 
#We can also use:
intersect(names(US_poverty),names(US_unemployment))

#which shows that there are two common column names 
#looking at the data, we can see this is because some common 
#column names have different capitalization. We could fix this by:

names(US_poverty)<- tolower(names(US_poverty))
names(US_unemployment)<- tolower(names(US_unemployment))


#notice that the % of distinct values in the common columns is highest
#in fipstxt which suggests this is likely to be the best column to
#use to merge the two datasets as 100% of the fipstxt values are unique
#relative the the number of rows. To merge the two we can then use:

US_county_poverty_and_unemployment_data<-left_join(US_unemployment,US_poverty)

#note that the left_join() command automatically finds the common
#column names and matches the two datasets based on this. 
#in practice this can be a better way of joining the two datasets as 
#it will do it's best to create a 'unique key' by matching across
#multiple columns. If we want to be specific about what to join by
#we can also specify the id using: 



#################################
#Is the data tidy? If not, what might we need to make it tidy?
#################################

#The data is not tidy as both the year and variable type are combined 
#eg unemployment_2004 specifies both the year and what is being measured 
#to fix this, we can use tidyr to reshape the data from wide to long


#reshape data from wide to long - option 1 - everything but - explicitly state names not to pivot
US_county_poverty_and_unemployment_data_long<-US_county_poverty_and_unemployment_data %>%
  pivot_longer(cols = -c("fipstxt","stabr","area_name"),
               names_to = "variable_year")


#reshape data from wide to long option 2 - pivot everything that ends with the number 2000 to 2019
US_county_poverty_and_unemployment_data_long_opt_2<-US_county_poverty_and_unemployment_data %>%
  pivot_longer(cols = ends_with(as.character(2000:2019)),
               names_to = "variable_year")


#reshape data from wide to long - option 3 - everything but, get names not to pivt from names() command
US_county_poverty_and_unemployment_data_long_opt_3<-US_county_poverty_and_unemployment_data %>%
  pivot_longer(cols = -c(names(US_county_poverty_and_unemployment_data)[1:3]),
               names_to = "variable_year")

#we can now get a summary of the different values in the variable_year column using
sort(unique(US_county_poverty_and_unemployment_data_long$variable_year))

#noting that in many cases the variable name simply includes identical
#measurement types over different time periods. As being able to 
#undertake analysis based on the year can be useful, we can use seperate()
#to create two columns from this using seperate(): 

US_county_poverty_and_unemployment_data_long<-separate(US_county_poverty_and_unemployment_data_long,
                                                       col=variable_year,
                                                       into=c('variable','year'), 
                                                       sep = -4)

#note that we specify 'sep = -4' to tell the function to sepearte
#based on the last 4 values in the variable_year column (as this is the year)
#We can also do this using a character, but in our case don't as "_" appears twice 
#If you test out replacing -4 with "_" it will split the column
#incorrectly. We can now view the variable names to see the result:

sort(unique(US_county_poverty_and_unemployment_data_long$variable))

#we can clean this up using stringr by replacing "_" with spaces and
#then feed the results into trimws() to remove trailing whitespace:
US_county_poverty_and_unemployment_data_long$variable<- US_county_poverty_and_unemployment_data_long$variable %>% 
  str_replace_all(pattern="_",replacement=" ") %>% 
  trimws()

#viewing this again, 
sort(unique(US_county_poverty_and_unemployment_data_long$variable))

#reshaping data so variables are 'wide' and years are 'long'
tidy_US_data<-US_county_poverty_and_unemployment_data_long %>% 
  pivot_wider(names_from="variable")


#View column names 
names(tidy_US_data)


#Note many variables now include spaces
#Although R can work with column names with special characters, 
#it can cause some complications down the line
#so it's a good idea to clean the column names
#You do this in R using 'make.names() eg: 

make.names(names(tidy_US_data))

#noting there are duplicate column names, we choose to
#specify that the names must be unique when we 
#overwrite current names 
names(tidy_US_data) <- make.names(names(tidy_US_data),unique=TRUE)

#noting the year column is stored as character, we can 
#convert it to a number by using as.integer() so R knows how to order it

tidy_US_data$year_int<-as.integer(tidy_US_data$year)



#check if it worked
table(tidy_US_data$year,tidy_US_data$year_int)



#############################
#Is the data ready for us to produce summaries? where is data missing?
#############################

#A number of approaches can be useful to test whether the data appears accurate 
#While this can vary according to the data, often
#a good principle is to check whether results of 
#simple analysis match simple principles that we know
#are true. For instance in the case of our data this
#could be tested via simple logical tests such as
#testing whether poverty and unemployment rates are <100%
#ensuring there are no duplicate records and
#the checking whether manually calculated totals make sense

#Logic check - are there duplicate records that  might 
#result in double counting and erroneous statistical summaries?

#check whether there is only one observation per year:
table(table(tidy_US_data$fipstxt, tidy_US_data$year)==1)

#Logic check - are manually calculated civilian labor force numbers 
#consistent with what we would expect for the US?

#summarize the size of the labour force by location and year
#to benchmark against other sources 

civ_lab_force<-group_by(tidy_US_data, year, stabr) 
civ_lab_force<-summarise(civ_lab_force, 
                         civ_lab_force = sum(civilian.labor.force))
#sort from largest to smallest to highlight outliers 
civ_lab_force<-arrange(civ_lab_force,desc(civ_lab_force,desc=TRUE))

#viewing the resulting summary by stabr, the largest area
#'US' has a labour force of 163 million in 2019 which roughly matches
#US labour statistics https://www.bls.gov/news.release/empsit.t01.htm
#the stabr'CA' which is california was 39 million in 2019
#which seems particularly high given their population is ~ 39 million!
#is a little higher than this https://en.wikipedia.org/wiki/Demographics_of_California
#in fact, comparing with the BLS data on CA it's almost 
#double other estimates https://www.bls.gov/eag/eag.ca.htm
#suggesting there is a problem with the data
#looking at the data, it appears it includes both 
#state and county statistics, meaning the labour force population
#will be counted twice (once for the county and once for the state)
#to fix this, we can exclude the state and national summaries 
#by noting fipstxt ending with '000' are state summaries
#labelling them using: 

tidy_US_data$is_summary_record<-str_sub(tidy_US_data$fipstxt,3) == "000" 

#we can now repeat the code above and include the newly 
#created variable as a group 

civ_lab_force<-group_by(tidy_US_data, year, stabr,is_summary_record) 
civ_lab_force<-summarise(civ_lab_force, 
                         civ_lab_force = sum(civilian.labor.force))

#making it easy to compare summaries: 
civ_lab_force<-pivot_wider(civ_lab_force,
                           names_from = is_summary_record,
                           values_from = civ_lab_force)

#noting CA looks more reasonable, while the manually calculated
#civilian labour force population are close to the state summaries

civ_lab_force$pct_diff<-civ_lab_force[,"FALSE"] / civ_lab_force[,"TRUE"]

#viewing summary statistics suggests manually calculated
#figures now match in most cases. We can choose to drop 
#records where this doesn't 
summary(civ_lab_force$pct_diff)
table(civ_lab_force$pct_diff-0.999<0.0001)


#Logic check - are unemployment and poverty rates <100%?
summary(tidy_US_data$unemployment.rate)
summary(tidy_US_data$pctpovall)

table(tidy_US_data$unemployment.rate<=100)
table(tidy_US_data$pctpovall<=100)

#Summarizing and analysing the data ----
#(to answer the questions we want to pose)


###########################
#what is the distribution unemployment? 
###########################

ggplot(data=tidy_US_data,aes(x=unemployment.rate)) +
  geom_histogram()


###########################
#Plot US Unemployment rate by state over time
###########################
#retain only state /national summaries
tidy_US_data_states<-filter(tidy_US_data,is_summary_record==TRUE)

tidy_US_data_county<-filter(tidy_US_data,is_summary_record !=TRUE)


ggplot(data=tidy_US_data_states,aes(group=stabr,x=year_int,y=unemployment.rate,color=stabr))+
  geom_line()

###########################
#How has the distribution of unemployment changed over time?
###########################
ggplot(data=tidy_US_data_county,aes(x=unemployment.rate)) +
  geom_histogram() +
  facet_grid(year_int ~.)

#we could also use a boxplot
ggplot(data=tidy_US_data_county,aes(x=year,y=unemployment.rate)) +
  geom_boxplot()  
#note, we use year, not year_int so R knows we want it to treat
#year as groups for labeling 

###########################
#Do the states with higher unemployment have higher levels of poverty?
###########################

scatter<-ggplot(data= tidy_US_data_states, aes(x=unemployment.rate, y=pctpovall)) +
  geom_point()

scatter

#add axis limits 
scatter<-scatter + xlim(2,6)

scatter

#add labels 

scatter<-scatter+labs(title= "US Employment Rate and Poverty (2018)",
     x="2018 Unemployment rate (%)", y="2018 estimated poverty rate (%)")

scatter

#apply theme 

scatter<-scatter+theme_bw()

scatter

#full code: 
clean_scatter<-ggplot(data= tidy_US_data_states, aes(x=unemployment.rate, y=pctpovall)) +
  geom_point(size=0) +
  xlim(2,6) +
  labs(title= "US Employment Rate and Poverty (2018)",
       x="2018 Unemployment rate (%)", y="2018 estimated poverty rate (%)") +
  geom_text(aes(label=stabr)) +theme_bw()


clean_scatter




#Saving the results ----
#(to present to others and add to other documents)

#ggsave outputs the last plot that was produced 
ggsave("clean scatter.png",scale=2)

