#Setting the working directory ----
#(so R knows where to look for data and save files)
setwd("Where our data is")
#Installing and loading the necessary packages ----
#(so R can access the any additional functions needed for the next steps)

library(readr) #to import csv
library(dplyr) #for summarizing data 
library(ggplot2) #for plotting 
library(tidyr) #to reshape/pivot data 
library(stringr) # to manipulate strings with str_replace


#Importing, exploring and cleaning the data ----
#(to get the data in R an make sure the data is error free for analysis)

titanic_data<- read_csv("titanic3.csv")

##############################################
#Import and Merge the boat data 
################################################
#data based off: http://www.icyousee.org/titanic.html#life
titanic_boat_data<-read_csv("titanic boat data.csv")

names(titanic_boat_data)
names(titanic_data)

#to 'join' the two datasets R needs a common 'key' to match the rows. 
titanic_data<-left_join(titanic_data,titanic_boat_data)

#left_join() guesses this based on the column names, but we 
#can check where matches weren't made by seeing where data is missing
table(is.na(titanic_data$boat_capacity),titanic_data$boat)

#based on this, it appears in most cases where we know the boat number, 
#boat_capacity was able to be matched against the new dataset


###############################################
#Create better labels and correct the incorrectly labeled passengers 
###############################################

#Assign survival variable labels making them factors
titanic_data<- titanic_data %>% mutate(survived = factor(survived,
                                                         levels=c(0,1), 
                                                         labels=c("Didn't Survive","Surviv3d")))

#factors can be confusing when first starting with R
# but the key is to understand they are essentially numbers with a label
#we can see the labels or levels using
levels(titanic_data$survived)
#and change them using 
levels(titanic_data$survived)[2]<-"Survived"


#stringr allows us to more easily work with strings. In this case we'd like to 
#change Boats 'C D' to "C" by feeding string to str_replace() function: 
titanic_data$boat<-str_replace(titanic_data$boat, "C D","C")

#another option for making changes like this (if we know the rows) is 
titanic_data$boat[120:122]<-"Five"

#while this might seem finicky when you're used to 'point and click' 
#software, the approach has advantages as it explicitly records 
#the changes made to the dataset so it's easy to reproduce what was done (rather than hiding it)


#Summarizing and analysing the data ----
#(to answer the questions we want to pose)

############################################### 
#Plot the number of passengers per title 
##############################################

#from looking at the data, the surname is operated by ','
#while titles end with a '.'. We can exploit this fact and 
#split one column into many with tidyr's separate() function
titanic_data<-separate(data=titanic_data, 
                       col=name, 
                       sep=",|[.]", 
                       into=c("surname","title","first_last_and_pref_name"),
                       remove=FALSE)
#note that the function takes the data, a column name 
#and what you want to use to 'separate' the column
#in this case we've said "," OR ".". Because "." 
#is used as a special character in R, we surround it with 
#square brackets to ensure it's interpreted properly 
#note that the 'remove=FALSE' option allows us to keep 
#the original column we are splitting (for checking it worked)
#we also need to be aware that if the column includes more 'splits' 
#than we've defined new columns for, it will display an error (as it does here)

#now that we have the results we can plot this using ggplot2
#ggplot 2 works by combining the data with a 'geom' type 
#and a coordinate system 
#You can see this in the ggplot2 cheat sheet available in 
#Rstudio via Help -> Cheatsheets -> Data Visualisation with ggplot2

#Another way to think about this is that ggplot2 starts
#by adding layers to a canvas. The first step is therefore to 
#choose the dimensions of the canvas we need by
#specifying the data and relevant dimensions that will be visualized: 

plot_number_of_titles<-ggplot(titanic_data, 
                              aes(x=title))


#as we've stored the plot as an object, R won't display it unless we 
#enter the object name into the console. Once we've done this, 
#we can see the plot 'canvas' in Rstudio 
plot_number_of_titles


#However we also need to define the 'geom' or geometry 
#of our visualization (again see the cheat sheet for options)
#as we are interested in getting the 'count' of passengers
#for each title, we can use geom_barplot:

plot_number_of_titles+geom_bar()


#to make it prettier we could also add components such as a title:

plot_number_of_titles + 
  geom_bar() +
  labs(title="No. of Passengers by title",
       x="Passenger Title",
       y="No.")

#It's important to note that 'pipes' aren't naturally integrated into
#the way ggplot2 works so %>% and ggplots don't work well together

###############################################
#What was the average fare paid by title, Did this differ by class? What was the most common title?
###############################################



#We can now use group_by() and summarize()
#to see the number of passengers by title
#I'll use pipes to make the code shorter: 

fare_by_title<-titanic_data %>% 
  group_by(title,pclass,survived) %>% 
  summarise(n=n(),
            avg_fare_usd=mean(fare)*1.3)





############################################### 
#Plot the average fare by Title 
###############################################


#Create the canvas and store as object
plot_avg_fare_by_class<-ggplot(data=fare_by_title,
                               aes(y=avg_fare_usd,x=title))

#display the plot 
plot_avg_fare_by_class

#we now define the geom and overwrite the object. Note that 
#We use 'stat="identity"' as by default geom_bar() will show 
#the count of whatever the x variable is. This overrides
#the default and plots what is in y

plot_avg_fare_by_class<-plot_avg_fare_by_class +
  geom_bar(stat="identity")

#display the plot 
plot_avg_fare_by_class

#add labels 

plot_avg_fare_by_class<-plot_avg_fare_by_class + 
  labs(title="Average Fare by Passenger Title",
      x="Passenger Title",
      y="Average Fare (USD)")
  
#display the plot 
plot_avg_fare_by_class

#Add a theme

plot_avg_fare_by_class<- plot_avg_fare_by_class + 
  theme_bw()


#display the plot 
plot_avg_fare_by_class


#ggplot has another useful component of the 'coordinates' system 
#which is 'faceting'. Which allows us to split the canvas
#by specific categories. In this case, we want to split by pclass
#We can do this by reusing the previous plot and adding: 

plot_avg_fare_by_class + 
  facet_grid(pclass ~ .)

#Note the 'formula' system for defining facets essentially asks us 
#to define a grid with the formula being: row ~ columns
#in the example we told ggplot that we want to display pclass as rows
#However, we could add an additional dimension such as whether a passenger survived

plot_avg_fare_by_class + 
  facet_grid(pclass ~ survived)


###############################################
#Determine whether the boat occupancy is related to passenger survival
###############################################


#create measure of boat occupancy 
titanic_data<-mutate(titanic_data, occupancy_pct = boat_occupancy/boat_capacity)


#summarise occupancy by class and sex
titanic_survival_class_occupancy<-titanic_data %>% 
  group_by(occupancy_pct,sex,pclass,survived) %>% 
  summarise(n=n())


#to develop a % survival rate we can create a 
#lower dimension table to use as a denominator to allow 
# us to calculate the % that survived by occupancy rate, class and sex

titanic_survival_class_occupancy_totals<-titanic_data %>% 
  group_by(sex,pclass) %>% 
  summarise(total=n())

#notice this creates a smaller table we can use as a denominator 
#merge table and calculate pct survival by occupancy 

titanic_survival_class_occupancy<-left_join(titanic_survival_class_occupancy,
                                            titanic_survival_class_occupancy_totals)
#calculate % by passenger type 

titanic_survival_class_occupancy<-mutate(titanic_survival_class_occupancy,
                                         pct=n/total)

#reshape data so we can compare the % that survived
#with those that didn't. We use pivot_wider() to make it 'wider'
#see here for a tutorial - https://tidyr.tidyverse.org/articles/pivot.html
#essentially we are interested in 'pivoting' the survived column 
#so we can view the % survival rate of each passenger to aid comparison

titanic_survival_class_occupancy_wide<-pivot_wider(titanic_survival_class_occupancy,
                                                   names_from = survived,
                                                   values_from = pct)
#we can then sort the dataset so that its ordered 
#by pclass, sex and the pct occupancy to try and 
#put more 'like' passengers closer together 

titanic_survival_class_occupancy_wide<-arrange(titanic_survival_class_occupancy_wide,pclass,sex,occupancy_pct)


#check the results using a simple scatter plot in ggplot
ggplot(data=titanic_survival_class_occupancy_wide,aes(y=Survived,x=occupancy_pct))+
  geom_point()

#doesn't provide a clear picture, what about if we color by gender?
ggplot(data=titanic_survival_class_occupancy_wide,aes(y=Survived,x=occupancy_pct,color=sex))+
  geom_point()

#still not clear, however, we can separate it further by splitting by class
ggplot(data=titanic_survival_class_occupancy_wide,aes(y=Survived,x=occupancy_pct,color=sex))+
  geom_point() +
  facet_grid(pclass~sex)

#even with splits there doesn't appear to be a 
#clear linear relationship between the two
#However, we can see that 1st class passengers
#Most likely to survive were in lower occupancy boats
#Looking at the titanic boat data we can see why this 
#is likely the case ~ most of the boats that were first 
#launched contained mainly first class passengers! 
#See data source here: http://www.icyousee.org/titanic.html#life


#Saving the results ----
#(to present to others and add to other documents)

###############################################
#Save the plot in the working directory
###############################################

#We can actually save the plot via Rstudio 
#by clicking 'Zoom' and 'Save As'. However, when 
#we're trying to produce a lot of plots it can be better to do
#this automatically using ggsave() which will save 
#a png file of the last plot into the working directory 

ggsave("pct survived by gender and class.png",scale = 2.5)

